#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>

#include <xc.h>
#include <xc_config.h>
#include <xc_funcs.h>
/* 
 * Test for the lda_xc_ksdt subroutine 
 * the goal is to reproduce the results in Table S1  
 * Phys. Rev. Lett. 112, 076403 (2014) supplement
 * 
 * Important: Fix the reduced t value in lda_xc_ksdt to one of 
 * those t values in the paper
 *
*/


int main()
{
  XC(func_type) xcfunctional;
  const FLOAT pi = 4.0*atan(1.0);
  const FLOAT onethird = 1.0/3.0;
  const FLOAT twothird = 2.0/3.0;

  FLOAT T;
  FLOAT rs[7]={ 1.0, 2.0, 4.0, 6.0, 8.0, 10.0, 40.0 };
  FLOAT rho[7][2];

  int SPIN;
  int order;

  FLOAT fxc,vxc,vxcs[2];
  FLOAT dfxcdt;
  FLOAT dfxcdrs; 
  FLOAT d2fxdt2;
  FLOAT d2fxcrsdt;
  FLOAT d2fxcdrs2;


  int ii;

  int i, vmajor, vminor, vmicro;

  xc_version(&vmajor, &vminor,&vmicro);
  printf("Libxc version: %d.%d.%d\n", vmajor, vminor, vmicro);


  SPIN=XC_UNPOLARIZED;
  T=0.0;

  XC(func_init)(&xcfunctional, XC_LDA_XC_KSDT, SPIN);
  XC(lda_xc_ksdt_set_params)(&xcfunctional, T);


  printf("          rs             T             fxc            vxc\n");
  printf("\nKSDT: UNPOLARIZED\n");


  for(ii=0; ii<7; ii++)
    {
     rho[ii][0] = 3.0/(4.0*pi*rs[ii]*rs[ii]*rs[ii]);
     rho[ii][1] = 0.0;
     XC(lda)( &xcfunctional, 1, rho[ii], &fxc, &vxc, NULL, NULL);
     printf("%14.4f %14.4f %14.4f %14.4f\n", rs[ii], T, fxc, vxc);
  }


  XC(lda_end)(&xcfunctional);


  printf("\n\n");
  printf("KSDT: POLARIZED\n");
  SPIN=XC_POLARIZED;
  T=100.0;

  XC(func_init)(&xcfunctional, XC_LDA_XC_KSDT, SPIN);
  XC(lda_xc_ksdt_set_params)(&xcfunctional, T);

  for(ii=0; ii<7; ii++)
    {
     rho[ii][0] = 3.0/(4.0*pi*rs[ii]*rs[ii]*rs[ii]);
     rho[ii][1] = 0.0;
     XC(lda)( &xcfunctional, 1, rho[ii], &fxc, vxcs, NULL, NULL);
     printf("%14.4f %14.4f %14.4f %14.4f\n", rs[ii], T, fxc, vxcs[0]);
    }

  XC(lda_end)(&xcfunctional);

}
